@@ -3,4 +3,4 @@
 # See file README.md or go to http://iic.jku.at/eda/research/quantum_verification/ for more information.
 #
 
-from .pyqfr import ConstructionMethod, construct, construct_grover, construct_qft, matrix_from_dd
+from mqt.qfr.pyqfr import ConstructionMethod, construct, construct_grover, construct_qft, matrix_from_dd
