@@ -96,18 +96,25 @@ namespace qc {
         //      `measure q[i] -> c[j];`
         // implies that the j-th (logical) output is obtained from measuring the i-th physical qubit.
         bool outputPermutationFound = !outputPermutation.empty();
+
+        // track whether the circuit contains measurements at the end of the circuit
+        // if it does, then all qubits that are not measured shall be considered garbage outputs
+        bool                outputPermutationFromMeasurements = false;
+        std::set<dd::Qubit> measuredQubits{};
+
         for (auto opIt = ops.begin(); opIt != ops.end(); ++opIt) {
             if ((*opIt)->getType() == qc::Measure) {
-                if (!isLastOperationOnQubit(opIt))
+                if (!isLastOperationOnQubit(opIt)) {
                     continue;
+                }
 
-                auto op = dynamic_cast<NonUnitaryOperation*>(opIt->get());
+                outputPermutationFromMeasurements = true;
+                auto op                           = dynamic_cast<NonUnitaryOperation*>(opIt->get());
                 assert(op->getTargets().size() == op->getClassics().size());
                 auto classicIt = op->getClassics().cbegin();
                 for (const auto& q: op->getTargets()) {
                     auto qubitidx = q;
                     auto bitidx   = *classicIt;
-
                     if (outputPermutationFound) {
                         // output permutation was already set before -> permute existing values
                         auto current = outputPermutation.at(qubitidx);
@@ -124,17 +131,33 @@ namespace qc {
                         // directly set permutation if none was set beforehand
                         outputPermutation[qubitidx] = static_cast<dd::Qubit>(bitidx);
                     }
+                    measuredQubits.emplace(qubitidx);
                     ++classicIt;
                 }
             }
         }
 
+        // clear any qubits that were not measured from the output permutation
+        // these will be marked garbage further down below
+        if (outputPermutationFromMeasurements) {
+            auto it = outputPermutation.begin();
+            while (it != outputPermutation.end()) {
+                const auto [physical, logical] = *it;
+                if (measuredQubits.find(physical) == measuredQubits.end()) {
+                    it = outputPermutation.erase(it);
+                } else {
+                    ++it;
+                }
+            }
+        }
+
         // if the output permutation is still empty, we assume the identity (i.e., it is equal to the initial layout)
         if (outputPermutation.empty()) {
             for (dd::QubitCount i = 0; i < nqubits; ++i) {
                 // only add to output permutation if the qubit is actually acted upon
-                if (!isIdleQubit(static_cast<dd::Qubit>(i)))
+                if (!isIdleQubit(static_cast<dd::Qubit>(i))) {
                     outputPermutation.insert({static_cast<dd::Qubit>(i), initialLayout.at(static_cast<dd::Qubit>(i))});
+                }
             }
         }
 
@@ -928,8 +951,10 @@ namespace qc {
     }
 
     std::ostream& QuantumComputation::print(std::ostream& os) const {
+        const auto width = ops.empty() ? 1 : static_cast<int>(std::log10(ops.size()) + 1.);
         if (!ops.empty()) {
-            os << std::setw((int)std::log10(ops.size()) + 1) << "i: \t\t\t";
+            os << std::setw(width) << "i"
+               << ": \t\t\t";
         } else {
             os << "i: \t\t\t";
         }
@@ -940,24 +965,26 @@ namespace qc {
                 os << static_cast<std::size_t>(Q.second) << "\t";
         }
         os << std::endl;
-        size_t i = 0;
+        size_t i = 0U;
         for (const auto& op: ops) {
-            os << std::setw((int)std::log10(ops.size()) + 1) << ++i << ": \t";
+            os << std::setw(width) << ++i << ": \t";
             op->print(os, initialLayout);
             os << std::endl;
         }
         if (!ops.empty()) {
-            os << std::setw((int)std::log10(ops.size()) + 1) << "o: \t\t\t";
+            os << std::setw(width) << "o"
+               << ": \t\t\t";
         } else {
             os << "o: \t\t\t";
         }
         for (const auto& physical_qubit: initialLayout) {
             auto it = outputPermutation.find(physical_qubit.first);
             if (it == outputPermutation.end()) {
-                if (garbage[physical_qubit.second])
+                if (garbage[physical_qubit.second]) {
                     os << "\033[31m|\t\033[0m";
-                else
+                } else {
                     os << "|\t";
+                }
             } else {
                 os << static_cast<std::size_t>(it->second) << "\t";
             }
